//	BLAlarm.cc
/*
This source file is part of G4beamline, http://g4beamline.muonsinc.com
Copyright (C) 2003,2004,2005,2006 by Tom Roberts, all rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

http://www.gnu.org/copyleft/gpl.html
*/

#ifndef BLALARM_HH
#define BLALARM_HH

#ifdef WIN32

class BLAlarm {
public:
	static void clear() { }
	static void set(int seconds) { }
	static void init() { }
};

#else

#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include "globals.hh"

/**	class BLAlarm implements an alarm clock.
 *	used to prevent an infinite tracking loop from hanging an entire job.
 *
 *	NOTE: all functions are inline, so no .cc file is needed.
 **/
class BLAlarm {
public:
	/// clear will clear any alarm. 
	static void clear() { alarm(0); }

	/// set() sets an alarm in the future.
	/// Implicitly clears any previous alarm.
	static void set(int seconds) { alarm(seconds); }

	/// init() will setup the signal handler
	/// Must be called before set().
	static void init() { signal(SIGALRM,sighandler); }

private:
	/// sighandler() handles the alarm signal
	static void sighandler(int sig) { 
		G4Exception("BLAlarm","Alarm Signal",FatalException,
			"SIGALRM fired, cannot be recovered");
	}
};

#endif // ! WIN32
#endif // BLALARM_HH
