#ifndef MICEPhysicsList_h
#define MICEPhysicsList_h 1

#include "G4VUserPhysicsList.hh"
#include "globals.hh"
#include "G4RegionStore.hh"

#include "G4MultipleScattering.hh"
#include "G4Decay.hh"

#include "G4HadronElasticProcess.hh"  // Hadron nuclear elastic scattering
#include "G4HadronCaptureProcess.hh"  // Capture of neutral hadrons in nuclei

// photons
#include "G4ComptonScattering.hh"
#include "G4GammaConversion.hh"
#include "G4PhotoElectricEffect.hh"

#include "G4Cerenkov.hh"
// #include "G4Scintillation.hh"
#include "G4OpAbsorption.hh"
// #include "G4OpRayleigh.hh"
// #include "G4OpBoundaryProcess.hh"

// electrons
#include "G4eIonisation.hh"
#include "G4eBremsstrahlung.hh"
#include "G4eplusAnnihilation.hh"

// muons
#include "G4MuIonisation.hh"
#include "G4MuBremsstrahlung.hh"
#include "G4MuPairProduction.hh"

// hadrons
#include "G4hIonisation.hh"
#include "G4PionPlusInelasticProcess.hh"
#include "G4PionMinusInelasticProcess.hh"
#include "G4ProtonInelasticProcess.hh"
#include "G4AntiProtonInelasticProcess.hh"
#include "G4NeutronInelasticProcess.hh"

// Low-energy Models
#include "G4LElastic.hh"  // LowE Hadron nuclear elastic scattering   
#include "G4LCapture.hh"  // LowE Capture of neutral hadrons in nuclei

#include "G4LEPionPlusInelastic.hh"
#include "G4LEPionMinusInelastic.hh"
#include "G4LEProtonInelastic.hh"
#include "G4LEAntiProtonInelastic.hh"
#include "G4LENeutronInelastic.hh"

#include "G4LowEnergyCompton.hh"
#include "G4LowEnergyBremsstrahlung.hh"
#include "G4LowEnergyIonisation.hh"
#include "G4LowEnergyRayleigh.hh"
#include "G4LowEnergyGammaConversion.hh"
#include "G4LowEnergyPhotoElectric.hh"

// Stopping processes
//#include "G4AntiProtonAnnihilationAtRest.hh"

class G4PhotoElectricEffect;
class G4ComptonScattering;
class G4GammaConversion;
// class G4Cerenkov;
// class G4Scintillation;
// class G4OpAbsorption;
// class G4OpRayleigh;
// class G4OpBoundaryProcess;

class G4LowEnergyCompton;
class G4LowEnergyRayleigh;
class G4LowEnergyGammaConversion;
class G4LowEnergyPhotoElectric;

class G4MultipleScattering;

class G4eIonisation;
class G4LowEnergyIonisation;
class G4eBremsstrahlung;
class G4LowEnergyBremsstrahlung;
class G4eplusAnnihilation;

class G4MuIonisation;
class G4MuEnergyLoss;
class G4MuBremsstrahlung;
class G4MuPairProduction;

class G4hIonisation;

//class G4AntiProtonAnnihilationAtRest;

class MICEPhysicsList: public G4VUserPhysicsList
{
  public:
    MICEPhysicsList();
    virtual ~MICEPhysicsList();
    void ConstructProcessesForOneParticle(G4ParticleDefinition *particle);

  protected:
    // Construct particle and physics
    virtual void ConstructParticle();
    virtual void ConstructProcess();

    virtual void SetCuts();

   
  protected:
    // these methods Construct particles 
    virtual void ConstructBosons();
    virtual void ConstructLeptons();
    virtual void ConstructMesons(); // uncommented by Rikard
    virtual void ConstructBaryons();
    virtual void ConstructIons();

  protected:
  // these methods Construct physics processes and register them
    virtual void ConstructAll();
    //virtual void ConstructOp();
    
  private:
    // gamma
    G4PhotoElectricEffect* thePhotoElectricEffect;
    G4ComptonScattering*   theComptonScattering;
    G4GammaConversion*     theGammaConversion;
 
//     G4Cerenkov*          theCerenkovProcess;
//     G4Scintillation*     theScintillationProcess;
//     G4OpAbsorption*      theAbsorptionProcess;
//     G4OpRayleigh*        theRayleighScatteringProcess;
//     G4OpBoundaryProcess* theBoundaryProcess;
    
    // e-
    G4MultipleScattering*  theeminusMultipleScattering;
    G4eIonisation*         theeminusIonisation;
    G4eBremsstrahlung*     theeminusBremsstrahlung;
    
    // e+
    G4MultipleScattering*  theeplusMultipleScattering;
    G4eIonisation*         theeplusIonisation;
    G4eBremsstrahlung*     theeplusBremsstrahlung;
    G4eplusAnnihilation*   theeplusAnnihilation;
    
    // mu+/-
    G4MuIonisation*        theMuMinusIonisation;
    G4MuIonisation*        theMuPlusIonisation;
    
    G4MultipleScattering*        theMuMinusMultScat;
    G4MultipleScattering*        theMuPlusMultScat;
    
    // pi+
    G4PionPlusInelasticProcess* thePionPlusInelastic;
    G4LEPionPlusInelastic* theLEPionPlusModel;
    G4MultipleScattering* thePionPlusMult;
    //G4hIonisation* thePionPlusIonisation;
    
    // pi-
    G4PionMinusInelasticProcess* thePionMinusInelastic;
    G4LEPionMinusInelastic* theLEPionMinusModel;
    G4MultipleScattering* thePionMinusMult;
    //G4hIonisation* thePionMinusIonisation;
       
    // proton
    G4ProtonInelasticProcess* theProtonInelastic;
    G4LEProtonInelastic* theLEProtonModel;
    G4MultipleScattering* theProtonMult;
    //G4hIonisation* theProtonIonisation;

    // anti-proton
    G4AntiProtonInelasticProcess* theAntiProtonInelastic;
    G4LEAntiProtonInelastic* theLEAntiProtonModel;
    G4MultipleScattering* theAntiProtonMult;
    //G4hIonisation* theAntiProtonIonisation;
    //G4AntiProtonAnnihilationAtRest* theAntiProtonAnnihilation;
  
    // neutron
    G4NeutronInelasticProcess*  theNeutronInelastic;
    G4LENeutronInelastic* theLENeutronModel;
    G4HadronCaptureProcess*  theNeutronCapture;
    G4LCapture* theNeutronCaptureModel;
        
   public:
   
   inline G4MuIonisation* theMuMinusIonisationf() 
                         {return theMuMinusIonisation; }
   inline G4MuIonisation* theMuPlusIonisationf() 
                         {return theMuPlusIonisation; }
			 
   inline G4MultipleScattering* theMuMinusMultScatf() 
                         {return theMuMinusMultScat; }
   inline G4MultipleScattering* theMuPlusMultScatf() 
                         {return theMuPlusMultScat; }
			 
};

#endif



