//	BLCMDeventcuts.cc
/*
This source file is part of G4beamline, http://g4beamline.muonsinc.com
Copyright (C) 2003,2004,2005,2006 by Tom Roberts, all rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

http://www.gnu.org/copyleft/gpl.html
*/

#include <stdio.h>

#include "BLCommand.hh"
#include "BLManager.hh"

class BLCMDeventcuts : public BLCommand {
	G4String filename;
public:
	BLCMDeventcuts();

	G4String commandName() { return "eventcuts"; }
	
	int command(BLArgumentVector& argv, BLArgumentMap& namedArgs);

	void defineNamedArgs();
};

BLCMDeventcuts defaultEventcutsCommand;

BLCMDeventcuts::BLCMDeventcuts()
{
	registerCommand(BLCMDTYPE_CUTS);
	setSynopsis("implements cuts on event number.");
	setDescription("The file is ASCII, with one event number per line. "
		"Events whose EventID is not contained in the file will be "
		"skipped. "
		"The file is read at the start of every run, so it can be "
		"changed between runs. This is particularly useful when "
		"the file is written by HistoRoot (via its EventID option), "
		"because the G4beamline GUI "
		"and a HistoRoot window can both be left open, and the user "
		"can alternate between them, selecting regions of plots with "
		"HistoScope sliders, and then looking at the selected events "
		"with G4beamline and a viewer.");

	filename = "";
}

int BLCMDeventcuts::command(BLArgumentVector& argv, BLArgumentMap& namedArgs)
{
	int retval = handleNamedArgs(namedArgs);

	BLManager::getObject()->setEventCutFile(filename);

	print("");

	return retval;
}

void BLCMDeventcuts::defineNamedArgs()
{
	argString(filename,"filename","The file containing a list of event numbers.");
	argString(filename,"file","Synonym for filename.");
}
