//	BLVisManager.cc
/*
This source file is part of G4beamline, http://g4beamline.muonsinc.com
Copyright (C) 2003,2004,2005,2006 by Tom Roberts, all rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

http://www.gnu.org/copyleft/gpl.html
*/
//
//	Copied from ExN03VisManager.cc, and extended.
//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: BLVisManager.cc,v 1.4 2005/08/20 21:11:00 tjrob Exp $
// GEANT4 tag $Name:  $
//
//
// John Allison 24th January 1998.

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifdef G4VIS_USE

#include <stdlib.h>
#include <stdio.h>

#include "BLVisManager.hh"
#include "BLManager.hh"
#include "BLCommand.hh"
#include "BLLoad.hh"

// Supported drivers...

// Not needing external packages or libraries...
#include "G4ASCIITree.hh"
#include "G4DAWNFILE.hh"
#include "G4HepRepFile.hh"
#include "G4HepRep.hh"
#include "G4RayTracer.hh"
#include "G4VRML1File.hh"
#include "G4VRML2File.hh"

// Needing external packages or libraries...

#ifdef G4VIS_USE_DAWN
#include "G4FukuiRenderer.hh"
#endif

#ifdef G4VIS_USE_OPACS
#include "G4Wo.hh"
#include "G4Xo.hh"
#endif

#ifdef G4VIS_USE_OPENGLX
#include "G4OpenGLImmediateX.hh"
#include "G4OpenGLStoredX.hh"
#endif

#ifdef G4VIS_USE_OPENGLWIN32
#include "G4OpenGLImmediateWin32.hh"
#include "G4OpenGLStoredWin32.hh"
#endif

#ifdef G4VIS_USE_OPENGLXM
#include "G4OpenGLImmediateXm.hh"
#include "G4OpenGLStoredXm.hh"
#endif

#ifdef G4VIS_USE_OPENGLQT
#include "G4OpenGLImmediateQt.hh"
#include "G4OpenGLStoredQt.hh"
#endif

#ifdef G4VIS_USE_OIX
#include "G4OpenInventorX.hh"
#endif

#ifdef G4VIS_USE_OIWIN32
#include "G4OpenInventorWin32.hh"
#endif

#ifdef G4VIS_USE_VRML
#include "G4VRML1.hh"
#include "G4VRML2.hh"
#endif


BLVisManager *BLVisManager::manager = 0;

#ifdef STUB
__attribute__((constructor)) void startup()
{
	BLVisManager *p = new BLVisManager(Param.getString("viewer"));
	visManager->init();
}
#endif

BLVisManager::BLVisManager(G4String _viewer) :
	G4VisManager(), BLManager::RunAction(), initCommands(), beginRunCommands(),
	endRunCommands()
{
	viewer = _viewer; 
	UI = G4UImanager::GetUIpointer();
	manager = this;

	// read VISUAL_DEF_FILENAME, and find the section for our viewer
	readSection(viewer);
}

void BLVisManager::init()
{
	Initialize();	// calls RegisterGraphicsSystems()

	printf("\nSelected visualization viewer: %s\n",viewer.c_str());

	// perform the init commands
	for(unsigned int i=0; i<initCommands.size(); ++i)
		UI->ApplyCommand(initCommands[i]);
	fputs("\n",stdout);

	// register us for UserRunAction
	BLManager::getObject()->registerRunAction(this,false);
}

void BLVisManager::readSection(G4String section)
{
	FILE *in = fopen(VISUAL_DEF_FILENAME,"r");
	if(!in) {
		char *g4bl_dir = getenv("G4BL_DIR");
		if(g4bl_dir) {
			G4String name(g4bl_dir);
			name += "/" VISUAL_DEF_FILENAME;
			in = fopen(name.c_str(),"r");
		}
		if(!in) {
			G4Exception("BLVisManager","Cannot read " 
					VISUAL_DEF_FILENAME,FatalException, "");
		}
	}
	G4String seek = "["; {seek += section; seek += "]";}
	char line[256];
	for(;;) {
		if(!fgets(line,sizeof(line),in)) {
			G4Exception("BLVisManager","Viewer not defined" 
					,FatalException, viewer.c_str());
		}
		if(line[0] == '#' || line[0] == '\n') continue;
		if(strncmp(line,seek.c_str(),seek.size()) == 0) break;
	}

	// Now read our commands into the command vectors
	while(fgets(line,sizeof(line),in)) {
		if(line[0] == '#' || line[0] == '\n') continue;
		if(line[0] == '[') break;
		char *p = strchr(line,'\n'); // remove newline
		if(p) *p = '\0';
		if(strncmp(line,"init:",5) == 0) {
			initCommands.push_back(G4String(line+5));
		} else if(strncmp(line,"include:",8) == 0) {
			readSection(line+8);
		} else if(strncmp(line,"help:",5) == 0) {
			puts(line+5);
		} else if(strncmp(line,"load:",5) == 0) {
			if(BLLoad::load(line+5))
				printf("BLVisManager: loaded '%s'\n",line+5);
			else
				BLCommand::printError("BLVisManager: Cannot "
					"load '%s'",line+5);
		} else if(strncmp(line,"beginRun:",9) == 0) {
			beginRunCommands.push_back(G4String(line+9));
		} else if(strncmp(line,"endRun:",7) == 0) {
			endRunCommands.push_back(G4String(line+7));
		} else {
			printf("BLVisManager::init invalid command for viewer"
				"'%s':\n\t%s\n",viewer.c_str(),line);
		}
	}

	fclose(in);
}

void BLVisManager::BeginOfRunAction(const G4Run *run)
{
	for(unsigned int i=0; i<beginRunCommands.size(); ++i)
		UI->ApplyCommand(beginRunCommands[i]);
}

void BLVisManager::EndOfRunAction(const G4Run *run)
{
	for(unsigned int i=0; i<endRunCommands.size(); ++i)
		UI->ApplyCommand(endRunCommands[i]);
}


void BLVisManager::RegisterGraphicsSystems () 
{
  // Graphics Systems not needing external packages or libraries...
  RegisterGraphicsSystem (new G4ASCIITree);
  RegisterGraphicsSystem (new G4DAWNFILE);
  RegisterGraphicsSystem (new G4HepRepFile);
  RegisterGraphicsSystem (new G4HepRep);
  RegisterGraphicsSystem (new G4RayTracer);
  //RegisterGraphicsSystem (new G4VRML1File);
  //RegisterGraphicsSystem (new G4VRML2File);

  // Graphics systems needing external packages or libraries...

#ifdef G4VIS_USE_DAWN
  RegisterGraphicsSystem (new G4FukuiRenderer);
#endif

#ifdef G4VIS_USE_OPACS
  RegisterGraphicsSystem (new G4Wo);
  RegisterGraphicsSystem (new G4Xo);
#endif

#ifdef G4VIS_USE_OPENGLX
  RegisterGraphicsSystem (new G4OpenGLImmediateX);
  RegisterGraphicsSystem (new G4OpenGLStoredX);
#endif

#ifdef G4VIS_USE_OPENGLWIN32
  RegisterGraphicsSystem (new G4OpenGLImmediateWin32);
  RegisterGraphicsSystem (new G4OpenGLStoredWin32);
#endif

#ifdef G4VIS_USE_OPENGLXM
  RegisterGraphicsSystem (new G4OpenGLImmediateXm);
  RegisterGraphicsSystem (new G4OpenGLStoredXm);
#endif

#ifdef G4VIS_USE_OPENGLQT
  RegisterGraphicsSystem(new G4OpenGLImmediateQt);
  RegisterGraphicsSystem(new G4OpenGLStoredQt);
#endif

#ifdef G4VIS_USE_OIX
  RegisterGraphicsSystem (new G4OpenInventorX);
#endif

#ifdef G4VIS_USE_OIWIN32
  RegisterGraphicsSystem (new G4OpenInventorWin32);
#endif

#ifdef G4VIS_USE_VRML
  RegisterGraphicsSystem (new G4VRML1);
  RegisterGraphicsSystem (new G4VRML2);
#endif

  if (fVerbose > 0) {
    G4cout <<
      "\nYou have successfully chosen to use the following graphics systems."
	 << G4endl;
    PrintAvailableGraphicsSystems ();
  }
}

#endif
