#include "globals.hh"
#include "MICEPhysicsList.hh"

#include "G4ParticleDefinition.hh"
#include "G4ParticleWithCuts.hh"
#include "G4ProcessManager.hh"
#include "G4ProcessVector.hh"
#include "G4ParticleTypes.hh"
#include "G4ParticleTable.hh"
#include "G4Material.hh"
#include "G4MaterialTable.hh"
#include "G4UserSpecialCuts.hh"
//
// Fake out the MICE data cards
//#include "dataCards.hh"
#include "BLCommand.hh"
class DataCards {
public:
	DataCards() { }
	double fetchValueDouble(G4String name) {
		if(name == "ProductionThreshold")
			return 1.0 * mm;
		else if(name == "MuonHalfTime")
			return -1.0;	// use default
		else
			BLCommand::printError("Unknown fetchValueDouble(%s)\n",
								name.c_str());
		return 0.0;
	}
	G4String fetchValueString(G4String name) {
		if(name == "HadronDecayOption")
			return "Meson";
		else if(name == "HadronIonisationOption")
			return "Meson";
		else if(name == "HadronScatteringOption")
			return "Meson";
		else if(name == "BeamType")
			return "Muon";
		else if(name == "MuonDecay")
			return "True";
		else if(name == "TrackerOption")
			return "NONE";
		else if(name == "HadronDecayOption")
			return "All";
		else if(name == "HadronIonisationOption")
			return "All";
		else if(name == "HadronScatteringOption")
			return "All";
		else
			BLCommand::printError("Unknown fetchValueString(%s)\n",
								name.c_str());
		return "";
	}
};
static DataCards MyDataCards;

#include "G4ios.hh"
#include <iostream>

using std::cout;

MICEPhysicsList::MICEPhysicsList(): G4VUserPhysicsList(),
  thePhotoElectricEffect(NULL),theComptonScattering(NULL),
  theGammaConversion(NULL),
  theeminusMultipleScattering(NULL),theeminusIonisation(NULL),
  theeminusBremsstrahlung(NULL),
  theeplusMultipleScattering(NULL),theeplusIonisation(NULL),
  theeplusBremsstrahlung(NULL),
  theeplusAnnihilation(NULL)
{
//   theCerenkovProcess           = 0;
//   theScintillationProcess      = 0;
//   theAbsorptionProcess         = 0;
//   theRayleighScatteringProcess = 0;
//   theBoundaryProcess           = 0;
  defaultCutValue = MyDataCards.fetchValueDouble("ProductionThreshold")*mm;
//   SetVerboseLevel(1);
}

MICEPhysicsList::~MICEPhysicsList()
{
}

void MICEPhysicsList::ConstructParticle()
{
  // In this method, static member functions should be called
  // for all particles which you want to use.
  // This ensures that objects of these particle types will be
  // created in the program.

  ConstructBosons();
  ConstructLeptons();
  ConstructMesons();
  ConstructBaryons();
  ConstructIons();
}

void MICEPhysicsList::ConstructBosons()
{
  // gamma
  G4Gamma::GammaDefinition();
  // optical photon
  // G4OpticalPhoton::OpticalPhotonDefinition();

  // pseudo-particles
  G4Geantino::GeantinoDefinition();
  // G4ChargedGeantino::ChargedGeantinoDefinition();
}

void MICEPhysicsList::ConstructLeptons()
{
  // leptons
  //  e+/-
  G4Electron::ElectronDefinition();
  G4Positron::PositronDefinition();
  // mu+/-
  G4MuonPlus::MuonPlusDefinition();
  G4MuonMinus::MuonMinusDefinition();
  // Neutrinos, needed for pion and mu decay.
  G4NeutrinoMu::NeutrinoMuDefinition();
  G4AntiNeutrinoMu::AntiNeutrinoMuDefinition();
  G4NeutrinoE::NeutrinoEDefinition();
  G4AntiNeutrinoE::AntiNeutrinoEDefinition();

}

void MICEPhysicsList::ConstructMesons()
{
  G4PionPlus::PionPlusDefinition();
  G4PionMinus::PionMinusDefinition();
  G4PionZero::PionZeroDefinition();
}

void MICEPhysicsList::ConstructBaryons()
{
  G4Proton::ProtonDefinition();
  G4AntiProton::AntiProtonDefinition();
  G4Neutron::NeutronDefinition();
}

void MICEPhysicsList::ConstructIons()
{
  G4GenericIon::GenericIonDefinition();
}

void MICEPhysicsList::ConstructProcess()
{
  AddTransportation();
  ConstructAll();
  //ConstructOp();
}

void MICEPhysicsList::ConstructAll()
{
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    ConstructProcessesForOneParticle(particle);
  }
}

void MICEPhysicsList::ConstructProcessesForOneParticle(G4ParticleDefinition *particle)
{
    G4ProcessManager* pmanager = particle->GetProcessManager();
    G4String particleName = particle->GetParticleName();
    G4String particleType = particle->GetParticleType();
    //G4cout << particleType << G4endl;

      if (particleName == "gamma") {
        thePhotoElectricEffect = new G4PhotoElectricEffect();
        theComptonScattering = new G4ComptonScattering();
        theGammaConversion = new G4GammaConversion();
        pmanager->AddDiscreteProcess(thePhotoElectricEffect);
        pmanager->AddDiscreteProcess(theComptonScattering);
        pmanager->AddDiscreteProcess(theGammaConversion);

        if (MyDataCards.fetchValueString("BeamType") == "Electron") {
          G4LowEnergyCompton *alComp = new G4LowEnergyCompton();
          G4LowEnergyRayleigh *alRay = new G4LowEnergyRayleigh();
          G4LowEnergyGammaConversion *alGamConv =
                                      new G4LowEnergyGammaConversion();
          G4LowEnergyPhotoElectric *alPE = new G4LowEnergyPhotoElectric();
          pmanager->AddDiscreteProcess(alComp );
          pmanager->AddDiscreteProcess(alRay);
          pmanager->AddDiscreteProcess(alGamConv);
          pmanager->AddDiscreteProcess(alPE);
        }
      } else if (particleName == "e-") {
        theeminusMultipleScattering = new G4MultipleScattering();
        pmanager->AddProcess(theeminusMultipleScattering);
        theeminusIonisation = new G4eIonisation();
        theeminusBremsstrahlung = new G4eBremsstrahlung();

        pmanager->AddProcess(theeminusIonisation);
        pmanager->AddProcess(theeminusBremsstrahlung);
        if (MyDataCards.fetchValueString("BeamType") == "Electron"){
          G4LowEnergyBremsstrahlung *alEBrem = new  G4LowEnergyBremsstrahlung();
          G4LowEnergyIonisation *alEIon = new G4LowEnergyIonisation();
          pmanager->AddProcess(alEIon);
          pmanager->AddProcess( alEBrem);
        }
        pmanager->SetProcessOrdering(theeminusMultipleScattering, idxAlongStep,1);
        pmanager->SetProcessOrdering(theeminusBremsstrahlung, idxAlongStep, 2);
        pmanager->SetProcessOrdering(theeminusIonisation, idxAlongStep, 3);
        pmanager->SetProcessOrdering(theeminusMultipleScattering, idxPostStep, 1);
        pmanager->SetProcessOrdering(theeminusBremsstrahlung, idxPostStep, 2);
        pmanager->SetProcessOrdering(theeminusIonisation, idxPostStep, 3);
      } else if (particleName == "e+") {
        theeplusMultipleScattering = new G4MultipleScattering();
        theeplusIonisation = new G4eIonisation();
        theeplusBremsstrahlung = new G4eBremsstrahlung();
        theeplusAnnihilation = new G4eplusAnnihilation();
        pmanager->AddProcess(theeplusMultipleScattering);
        pmanager->AddProcess(theeplusIonisation);
        pmanager->AddProcess(theeplusBremsstrahlung);
        pmanager->AddProcess(theeplusAnnihilation);
        pmanager->SetProcessOrderingToFirst(theeplusAnnihilation, idxAtRest);
        pmanager->SetProcessOrdering(theeplusMultipleScattering, idxAlongStep,  1);
        pmanager->SetProcessOrdering(theeplusIonisation, idxAlongStep,  2);
        pmanager->SetProcessOrdering(theeplusMultipleScattering, idxPostStep, 1);
        pmanager->SetProcessOrdering(theeplusIonisation, idxPostStep, 2);
        pmanager->SetProcessOrdering(theeplusBremsstrahlung, idxPostStep, 3);
        pmanager->SetProcessOrdering(theeplusAnnihilation, idxPostStep, 4);
      } else if( particleName == "mu+" || particleName == "mu-" ) {
        G4VProcess* aMultipleScattering;
        G4VProcess* aBremsstrahlung = new G4MuBremsstrahlung();
        G4VProcess* aPairProduction = new G4MuPairProduction();
        G4VProcess* anIonisation;
        if ( particleName == "mu+") {
          this->theMuPlusIonisation = new G4MuIonisation();
          anIonisation = this->theMuPlusIonisation;
          this->theMuPlusMultScat  = new G4MultipleScattering();
          aMultipleScattering = this->theMuPlusMultScat;
          //Magic Patch to fix multiple scale..
          //this->theMuPlusMultScat->SetScatteringParameter(scatParam);
        } else {
          this->theMuMinusIonisation = new G4MuIonisation();
          anIonisation = this->theMuMinusIonisation;
          this->theMuMinusMultScat  = new G4MultipleScattering();
          aMultipleScattering = this->theMuMinusMultScat;
          //Magic Patch to fix multiple scale..
          //this->theMuMinusMultScat->SetScatteringParameter(scatParam);
        }
        pmanager->AddProcess(anIonisation);
        pmanager->AddProcess(aMultipleScattering);
        pmanager->AddProcess(aBremsstrahlung);
        pmanager->AddProcess(aPairProduction);

        pmanager->SetProcessOrdering(aMultipleScattering, idxAlongStep, 1);
        pmanager->SetProcessOrdering(anIonisation, idxAlongStep,  2);

        pmanager->SetProcessOrdering(aMultipleScattering, idxPostStep, 1);
        pmanager->SetProcessOrdering(anIonisation, idxPostStep, 2);
        pmanager->SetProcessOrdering(aBremsstrahlung, idxPostStep, 3);
        pmanager->SetProcessOrdering(aPairProduction, idxPostStep, 4);

        if (MyDataCards.fetchValueString("MuonDecay") == "True"){
          // Allows tweaking muon decay, if turned on.
          G4double muHalfTime = MyDataCards.fetchValueDouble("MuonHalfTime")*nanosecond;
          if (muHalfTime > 0) { // else use g4 default
            particle->SetPDGLifeTime(muHalfTime);
          }
          G4VProcess* aDecay = new G4Decay();
          pmanager->AddProcess(aDecay);
          pmanager->SetProcessOrdering(aDecay, idxPostStep, 5);
          pmanager->SetProcessOrdering(aDecay, idxAtRest);
        }
      } else if (particleType == "meson") {   // Rikards own hadronics:
        G4HadronElasticProcess* theElasticProcess =
                                    new G4HadronElasticProcess;
        G4LElastic* theElasticModel = new G4LElastic;
        theElasticProcess->RegisterMe(theElasticModel);

        // Add decay
        //
        // Only charged pions are interesting. All other mesons are only allowed
        // to decay and create children which may or may not be subject to
        // ionisation etc.
        if ((MyDataCards.fetchValueString("HadronDecayOption") == "Meson")
          |(MyDataCards.fetchValueString("HadronDecayOption") == "All")){
          G4VProcess* aDecay = new G4Decay();
          pmanager ->AddProcess(aDecay);
          pmanager ->SetProcessOrdering(aDecay, idxPostStep);
          pmanager ->SetProcessOrdering(aDecay, idxAtRest);
        }

        // Add ionisation
        //
        // If the particle is charged allow it to ionize gas if if flag is set.
        if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Meson")
            |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
          if (particle->GetPDGCharge()){ //charged particle?
            G4hIonisation* theIonisation = new G4hIonisation();
            pmanager->AddProcess(theIonisation, ordInActive,2, 2);
          }
        }

        if (particleName == "pi+"){
          // PionPlus
          if ((MyDataCards.fetchValueString("HadronScatteringOption") == "Meson")
          |(MyDataCards.fetchValueString("HadronScatteringOption") == "All")){
            theLEPionPlusModel = new G4LEPionPlusInelastic();
            thePionPlusInelastic = new G4PionPlusInelasticProcess();
            thePionPlusInelastic->RegisterMe(theLEPionPlusModel);
            thePionPlusMult = new G4MultipleScattering();
            pmanager->AddDiscreteProcess(theElasticProcess);
            pmanager->AddDiscreteProcess(thePionPlusInelastic);
            pmanager->AddProcess(thePionPlusMult);
            pmanager->SetProcessOrdering(thePionPlusMult, idxAlongStep, 1);
            pmanager->SetProcessOrdering(thePionPlusMult, idxPostStep, 1);
          }
//           if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Meson")
//             |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
//             thePionPlusIonisation = new G4hIonisation();
//             pmanager->AddProcess(thePionPlusIonisation, ordInActive,2, 2);
//           }
        } else if (particleName == "pi-"){
          // PionMinus
          if ((MyDataCards.fetchValueString("HadronScatteringOption") == "Meson")
          |(MyDataCards.fetchValueString("HadronScatteringOption") == "All")){
            theLEPionMinusModel = new G4LEPionMinusInelastic();
            thePionMinusInelastic = new G4PionMinusInelasticProcess();
            thePionMinusInelastic->RegisterMe(theLEPionMinusModel);
            thePionMinusMult = new G4MultipleScattering();
            pmanager->AddDiscreteProcess(theElasticProcess);
            pmanager->AddDiscreteProcess(thePionMinusInelastic);
            pmanager->AddProcess(thePionMinusMult);
            pmanager->SetProcessOrdering(thePionMinusMult, idxAlongStep, 1);
            pmanager->SetProcessOrdering(thePionMinusMult, idxPostStep, 1);
          }
//           if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Meson")
//             |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
//             thePionMinusIonisation = new G4hIonisation();
//             pmanager->AddProcess(thePionMinusIonisation, ordInActive,2, 2);
//           }
        }

      } else if (particleType == "baryon") {
        G4HadronElasticProcess* theElasticProcess =
                                    new G4HadronElasticProcess;
        G4LElastic* theElasticModel = new G4LElastic;
        theElasticProcess->RegisterMe(theElasticModel);

        // Add decay
        //
        // Only protons and neutrons are interesting. All other baryons are
        // only allowed to decay and create children which may or may not be
        // subject to ionisation etc (if they can decay according to PDG).
        // Antiprotons and antineutrons are too rare.
        if ((MyDataCards.fetchValueString("HadronDecayOption") == "Baryon")
          |(MyDataCards.fetchValueString("HadronDecayOption") == "All")){
          G4VProcess* aDecay = new G4Decay();
          if (particle->GetPDGStable() == false){
            pmanager ->AddProcess(aDecay);
            pmanager ->SetProcessOrdering(aDecay, idxPostStep);
            pmanager ->SetProcessOrdering(aDecay, idxAtRest);
          }
        }

        // Add ionisation
        //
        // If the particle is charged allow it to ionize gas if if flag is set.
        if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Baryon")
            |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
          if (particle->GetPDGCharge()){ //charged particle?
            G4hIonisation* theIonisation = new G4hIonisation();
            pmanager->AddProcess(theIonisation, ordInActive,2, 2);
          }
        }

        if (particleName == "proton"){
          // Proton
          if ((MyDataCards.fetchValueString("HadronScatteringOption") == "Baryon")
          |(MyDataCards.fetchValueString("HadronScatteringOption") == "All")){
            theLEProtonModel = new G4LEProtonInelastic();
            theProtonInelastic = new G4ProtonInelasticProcess();
            theProtonInelastic->RegisterMe(theLEProtonModel);
            theProtonMult = new G4MultipleScattering();
            pmanager->AddDiscreteProcess(theElasticProcess);
            pmanager->AddDiscreteProcess(theProtonInelastic);
            pmanager->AddProcess(theProtonMult); // this is defeated in competition
            //with the elastic and inelastic scattering.
            pmanager->SetProcessOrdering(theProtonMult, idxAlongStep, 1);
            pmanager->SetProcessOrdering(theProtonMult, idxPostStep, 1);
            pmanager->SetProcessOrdering(theElasticProcess, idxPostStep, 3);
            pmanager->SetProcessOrdering(theProtonInelastic, idxPostStep, 4);
          }
//           if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Baryon")
//             |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
//             theProtonIonisation = new G4hIonisation();
//             pmanager->AddProcess(theProtonIonisation);
//             //pmanager->AddProcess(theProtonIonisation, ordInActive,2, 2);
//             pmanager->SetProcessOrdering(theProtonIonisation, idxAlongStep, 2);
//             pmanager->SetProcessOrdering(theProtonIonisation, idxPostStep, 2);
//           }
        } else if (particleName == "anti_proton"){
          // Anti Proton
          if ((MyDataCards.fetchValueString("HadronScatteringOption") == "Baryon")
          |(MyDataCards.fetchValueString("HadronScatteringOption") == "All")){
            theLEAntiProtonModel = new G4LEAntiProtonInelastic();
            theAntiProtonInelastic = new G4AntiProtonInelasticProcess();
            theAntiProtonInelastic->RegisterMe(theLEAntiProtonModel);
            theAntiProtonMult = new G4MultipleScattering();
          //theAntiProtonAnnihilation =  new G4AntiProtonAnnihilationAtRest();
            pmanager->AddDiscreteProcess(theElasticProcess);
            pmanager->AddDiscreteProcess(theAntiProtonInelastic);
            pmanager->AddProcess(theAntiProtonMult);
            pmanager->SetProcessOrdering(theAntiProtonMult, idxAlongStep, 1);
            pmanager->SetProcessOrdering(theAntiProtonMult, idxPostStep, 1);
          //pmanager->AddRestProcess(theAntiProtonAnnihilation);
          }
//           if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Baryon")
//             |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
//             theAntiProtonIonisation = new G4hIonisation();
//             pmanager->AddProcess(theAntiProtonIonisation, ordInActive,2, 2);
//           }
        } else if (particleName == "neutron"){
          if ((MyDataCards.fetchValueString("HadronScatteringOption") == "Baryon")
          |(MyDataCards.fetchValueString("HadronScatteringOption") == "All")){
            theLENeutronModel = new G4LENeutronInelastic();
            theNeutronInelastic = new G4NeutronInelasticProcess();
            theNeutronInelastic->RegisterMe(theLENeutronModel);
            pmanager->AddDiscreteProcess(theElasticProcess);
            pmanager->AddDiscreteProcess(theNeutronInelastic);
          }
          theNeutronCaptureModel = new G4LCapture();
          theNeutronCapture = new G4HadronCaptureProcess();
          theNeutronCapture->RegisterMe(theNeutronCaptureModel);
          pmanager->AddDiscreteProcess(theNeutronCapture);
        }
      }

      // Ions, also uncharged nucleus.
      if (particleType == "nucleus") {
        G4HadronElasticProcess* theElasticProcess =
                                    new G4HadronElasticProcess;
        G4LElastic* theElasticModel = new G4LElastic;
        theElasticProcess->RegisterMe(theElasticModel);

        // Add decay
        if ((MyDataCards.fetchValueString("HadronDecayOption") == "Ion")
          |(MyDataCards.fetchValueString("HadronDecayOption") == "All")){
          G4VProcess* aDecay = new G4Decay();
          if (particle->GetPDGStable() == false){
            pmanager ->AddProcess(aDecay);
            pmanager ->SetProcessOrdering(aDecay, idxPostStep);
            pmanager ->SetProcessOrdering(aDecay, idxAtRest);
          }
        }

        // Add ionisation
        //
        // If the nucleus is charged allow it to ionize gas if if flag is set.
        if ((MyDataCards.fetchValueString("HadronIonisationOption") == "Ion")
            |(MyDataCards.fetchValueString("HadronIonisationOption") == "All")){
          if (particle->GetPDGCharge()){ //charged particle?
            G4hIonisation* theIonisation = new G4hIonisation();
            pmanager->AddProcess(theIonisation, ordInActive,2, 2);
          }
        }
      }

}// end of method

//void MICEPhysicsList::ConstructOp()
//{
  //theCerenkovProcess = new G4Cerenkov();
  //theScintillationProcess = new G4Scintillation();
//   theAbsorptionProcess     = new G4OpAbsorption();
//   theRayleighScatteringProcess = new G4OpRayleigh();
//   theBoundaryProcess  = new G4OpBoundaryProcess();
//
//   theCerenkovProcess->SetMaxNumPhotonsPerStep(300);
//   theCerenkovProcess->SetTrackSecondariesFirst(true);
//
//   theScintillationProcess->SetScintillationYieldFactor(1.);
//   theScintillationProcess->SetTrackSecondariesFirst(true);
//
//   G4OpticalSurfaceModel themodel = unified;
//   theBoundaryProcess->SetModel(themodel);
//
//   theParticleIterator->reset();
//   while( (*theParticleIterator)() ){
//     G4ParticleDefinition* particle = theParticleIterator->value();
//     G4ProcessManager* pmanager = particle->GetProcessManager();
//     G4String particleName = particle->GetParticleName();
//     if (theCerenkovProcess->IsApplicable(*particle)) {
//       pmanager->AddContinuousProcess(theCerenkovProcess);
//     }
//     if (theScintillationProcess->IsApplicable(*particle)) {
//       pmanager->AddProcess(theScintillationProcess);
//       pmanager->SetProcessOrderingToLast(theScintillationProcess, idxAtRest);
//       pmanager->SetProcessOrderingToLast(theScintillationProcess, idxPostStep);
//     }
//     if (particleName == "opticalphoton") {
//       G4cout << " AddDiscreteProcess to OpticalPhoton " << G4endl;
//       pmanager->AddDiscreteProcess(theAbsorptionProcess);
//       pmanager->AddDiscreteProcess(theRayleighScatteringProcess);
//       pmanager->AddDiscreteProcess(theBoundaryProcess);
//     }
//   }
//}


void MICEPhysicsList::SetCuts()
{
  if (verboseLevel >1){
    G4cout << "MICEPhysicsList::SetCuts:";
  }
  //  " G4VUserPhysicsList::SetCutsWithDefault" method sets
  //   the default cut value for all particle types
  SetCutsWithDefault();


  // SetCutValue sets a cut value for a particle type for a region
  // Geant4 presently only supports gamma, e+ & e-.
  //SetCutValue(0.5*mm, "e-", const G4String& rname);
  typedef std::vector<G4Region*>::iterator regionIterator;
  G4RegionStore* regionStore = G4RegionStore::GetInstance();
  G4cout << "\nList of defined G4Regions: " << G4endl;
  for(regionIterator rItr=regionStore->begin();rItr!=regionStore->end();rItr++)
  {
    G4Region* aR = *rItr;
    G4cout << "- " << aR->GetName() << G4endl;
  }

  if (MyDataCards.fetchValueString("TrackerOption") == "TPG") {
    // This is the sensitive gas region of the TPG and the volumes inside it.
    G4Region* regA = G4RegionStore::GetInstance()->GetRegion("lGasVolumeTPGUp");
    G4Region* regB = G4RegionStore::GetInstance()->GetRegion("lGasVolumeTPGDown");
    G4ProductionCuts* cuts = new G4ProductionCuts;
    cuts->SetProductionCut(0.5*mm);
    regA->SetProductionCuts(cuts);
    regB->SetProductionCuts(cuts);
  }
  //SetCutValue(0.5*mm, "e-");



  DumpCutValuesTable();
}
